import logging

from google.cloud import pubsub_v1


class PubsubPublisher:
    def __init__(self, project_id: str, topic_id: str, enable_message_ordering=None):
        publisher_options = pubsub_v1.types.PublisherOptions(
            enable_message_ordering=enable_message_ordering
        )
        self.publisher = pubsub_v1.PublisherClient(publisher_options=publisher_options)
        self.topic_path = self.publisher.topic_path(project_id, topic_id)
        self._logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")

    def publish_and_wait_for_message_id(self, data: bytes, ordering_key=None, **attrs):
        future = self._publish(data, ordering_key, **attrs)
        message_id = future.result()
        self._logger.debug(f"Published data={data} > message_id={message_id} ")
        return message_id

    def publish(self, data: bytes, ordering_key=None, **attrs):
        future = self._publish(data, ordering_key, **attrs)
        self._logger.debug(f"Published data={data}")
        return future

    def _publish(self, data: bytes, ordering_key=None, **attrs):
        _attrs = {
            k: v for k, v in attrs.items() if v is not None and isinstance(v, str)
        }
        return self.publisher.publish(
            topic=self.topic_path,
            data=data,
            ordering_key=ordering_key if ordering_key else "",
            **_attrs,
        )



import logging
import sys
from typing import Callable

from google.cloud import pubsub_v1
from google.cloud.pubsub_v1.subscriber.message import Message


class PubsubConsumer:
    def __init__(
        self,
        project_id: str,
        subscription_id: str,
        callback: Callable[[Message], None],
        max_messages=1,
    ):
        if not project_id:
            raise ValueError("Missing project_id")
        if not subscription_id:
            raise ValueError("Missing subscription_id")
        if not callback:
            raise ValueError("Missing callback")
        self.subscriber = pubsub_v1.SubscriberClient()
        self.subscription_path = pubsub_v1.SubscriberClient.subscription_path(
            project_id, subscription_id
        )
        self.flow_control = pubsub_v1.types.FlowControl(max_messages=max_messages)
        self.callback = callback
        self.streaming_pull_future = None
        self._logger = logging.getLogger(f"{__name__}.{self.__class__.__name__}")

    def shutdown(self):
        if self.streaming_pull_future and self.streaming_pull_future.cancel:
            self._logger.debug(
                f"Stop listening to '{self.subscription_path}'. Shutting down..."
            )
            try:
                self.streaming_pull_future.cancel()  # Trigger the shutdown.
                self.streaming_pull_future.result()  # Block until the shutdown is complete.
                self._logger.debug(f"Shutdown")
            except:
                self._logger.error(f"Shutdown failed due to {sys.exc_info()}")
            finally:
                self.subscriber.close()

    def start(self):
        def _callback_wrapper(message: Message):
            self._logger.debug(
                f"Received"
                f" message_id={message.message_id}"
                f" data={message.data}"
                f" attributes={message.attributes}"
                f" publish_time={message.publish_time}"
                f" ordering_key={message.ordering_key}"
                f" delivery_attempt={message.delivery_attempt}"
            )
            self.callback(message)

        # https://cloud.google.com/pubsub/docs/samples/pubsub-subscriber-flow-settings#pubsub_subscriber_flow_settings-python
        # Limit the subscriber to only have N outstanding messages at a time.
        self.streaming_pull_future = self.subscriber.subscribe(
            self.subscription_path,
            callback=_callback_wrapper,
            flow_control=self.flow_control,
            await_callbacks_on_shutdown=True,
        )
        self._logger.debug(f"Start listening to '{self.subscription_path}'")
        return self.streaming_pull_future



import setuptools

# Read description
with open("USAGE.md", "r") as f:
    long_description = f.read()

with open("src/sparta/pubsub/VERSION", "r") as f:
    version = f.read().strip()

# Creates the setup config for the package
# We only look for code into the "Modules" folder
setuptools.setup(
    name="sparta-pubsub",
    version=version,
    author="Spartan Approach",
    author_email="sparta@spartanapproach.com",
    description="Sparta pubsub library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Spartan-Approach/sparta-pubsub",
    install_requires=[
        "google-cloud-pubsub",
    ],
    extras_require={
        "test": [
            "pytest",
            "pytest-asyncio",
            "python-dotenv",
        ],
    },
    packages=setuptools.find_packages(
        where="src", exclude=["*.tests", "*.tests.*", "tests.*", "tests"]
    ),
    package_dir={"": "src"},
    namespace_packages=["sparta"],
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",  # TODO SELECT A LICENSE
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
)

# Command to generate the distributions and the whl
# python3 setup.py sdist bdist_wheel
